Rem
Rem	UCI interface for Saitek-OSA-Boards (COM port)
Rem Optimized for Arena
Rem V1.3
Rem

#include "file.bi"

' Setup board position from FEN string
Declare Sub setuppos(ByRef fenstr As String)

'
'   Main program
'

DIM As UShort n, i
DIM As String indata, dat, cdata, oldFEN, bestmove, actmove, pv
DIM As String actCOM, actBaud
Dim As UByte flp, firstmove
Dim Shared black As UByte
Dim Shared nParams As UByte
ReDim Shared Param(1 To 2) As String
actCOM = "COM1:"
actBaud = "1200"
flp = 0
firstmove = 0
black = 0

' Checking for INI file
If FileExists("osa.ini") Then
    Open "osa.ini" for Input As #4
    Do Until EOF(4)
        Line Input #4, indata
        indata = Trim(indata) : dat = ""
        ' Strip off blanks
        For i = 1 to Len(indata)
            If Mid(indata, i, 1) <> " " Then dat = dat + Mid(indata, i, 1)
        Next i
        If LCase(Left(dat, 4)) = "com=" Then actCOM = Mid(dat, 5)
        If LCase(Left(dat, 5)) = "baud=" Then actBaud = Mid(dat, 6)
        If LCase(Left(dat, 5)) = "flip=" Then flp = ValInt(Mid(dat, 6))
    Loop
    Close #4                        
End If

' Checking for command line parameters
nParams = 0
While Command(nParams + 1) <> ""
    nParams = nParams + 1
Wend
If nParams > 0 Then
    ReDim Param(1 To nParams) As String
    For i = 1 To nParams
        Param(i) = Command(i)
    Next i
End If

' Help
If UCase(Param(1)) = "/?" Or UCase(Param(1)) = "/H" Then
    Print
    Print "Play with Scisys/Saitek-OSA-Boards under Arena"
    Print "(C) 2012 by Achim Pietig, Version 1.3"
    Print
    Print "Usage: OSA-Board [/?] Or [/h]"
    Print "       OSA-Board [COMx:] [baud]"
    Print "Call without parameters sets COM1: with 1200 baud"
    Print "Board shall be set to corresponding baud rate before!"
    Print
    Print "e.g.   OSA-Board                (COM1: with 1200 baud)"
    Print "       OSA-Board COM2:          (COM2: with 1200 baud)"
    Print "       OSA-Board COM3: 9600     (COM3: with 9600 baud)"
    End
End If

' Init program parameters
If nParams > 0 Then actCOM = UCase(Param(1))
If nParams > 1 Then actBaud = Param(2)

Open Cons For Input As #1
If Open Com (actCOM + actbaud + ",N,8,1,CS0,DS0,CD0,RS,BIN" For Binary As #2) <> 0 Then
    Print "Cannot initialise comport"
    Close
    End
End If

'debug
'Open "OSA_log.txt" For Output As #3

Print #2, "ENGLISH"
Sleep 250, 1

Print #2, "OPEN"
Do
    dat = Input(1, #2)
    If Asc(dat) > 31 And dat <> ">" Then cdata = cdata + dat
Loop Until dat = Chr(&H17)
Print "info string "; Trim(cdata)

Print #2, "SENDINFO OFF"
Do
    dat = Input(1, #2)
Loop Until dat = Chr(&H17)

Print #2, "BOARD ON"
Do
    dat = Input(1, #2)
Loop Until dat = Chr(&H17)

' Program loop
Do
        Line Input #1, indata
        indata = Trim(indata)
        cdata = "" : dat = ""

        ' Flush COM buffer
        While LOC(2) > 0
            dat = Input(LOC(2), #2)
        Wend

        Print #2, "NORMAL"
        Do
            dat = Input(1, #2)
        Loop Until dat = Chr(&H17)
        
        If indata = "uci" Then
            Print "id name OSA-Board V1.3"
            Print "id author Achim Pietig"
            Print "option name OwnBook type check default true"
            Print "uciok"
        End If

        If indata = "isready" Then
            Print "readyok"
        End If

        If indata = "ucinewgame" Then
            Print #2, "NEWGAME"
            Do
                dat = Input(1, #2)
            Loop Until dat = Chr(&H17)
            firstmove = 1
            black = 0
        End If

        If indata = "board" Then
            Print #2, "POSITION"
            Do
                dat = Input(1, #2)
                If Asc(dat) > 31 And dat <> ">" Then cdata = cdata + dat
                If Asc(dat) = 13 Then cdata = cdata + dat
                If Asc(dat) = 10 Then cdata = cdata + dat
            Loop Until dat = Chr(&H17)
            Print cdata
        End If

        If indata = "flip" Then
            flp = 1
        End If

        If Left(indata, 9) = "position " Then
            n = Instr(10, indata, "moves ") : actmove = "" : pv = ""
            If n > 0 Then
                pv = Mid(indata, n + 6)
                If Instr(1, "qrnb", Right(indata, 1)) > 0 Then
                    actmove = Right(indata, 5)
                Else
                    actmove = Right(indata, 4)
                End If
                actmove = Left(actmove, 2) + "-" + Mid(actmove, 3)
                If Instr(1, "qrnb", Right(actmove, 1)) > 0 Then actmove = Left(actmove, 5) + "/" + UCase(Right(actmove, 1))
                indata = Left(indata, n - 2)
            End If
            If Instr(10, indata, "startpos") And Len(pv) < 4 Then
                Print #2, "NEWGAME"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
				firstmove = 1
                black = 0
            End If
            n = Instr(10, indata, "fen ")
            If n > 0 Then
                cdata = Trim(Mid(indata, n + 4))
                If cdata = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w AHah -"  And Len(pv) < 4 Then
                    Print #2, "NEWGAME"
                    Do
                        dat = Input(1, #2)
                    Loop Until dat = Chr(&H17)
					firstmove = 1
                    black = 0
                Else
                    If cdata <> oldFEN Then
                        oldFEN = cdata
                        setuppos(cdata)
                    End If
                End If
            End If
        End If

        If Left$(indata, 2) = "go" Then
            If firstmove = 1 And flp = 1 And Len(actmove) > 4 Then
                firstmove = 0 : black = 1
                Print #2, "PLAY"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Sleep 250, 1
                Print #2, "BOARD OFF"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Sleep 250, 1
                Print #2, "TAKEBACK"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Sleep 250, 1
                Print #2, "MOVE "; actmove
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                actmove = ""
                Sleep 250, 1
                Print #2, "BOARD ON"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Sleep 250, 1
                Print #2, "NORMAL"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
            End If

            If Len(actmove) > 4 Then
                If firstmove = 1 Then black = 1 : firstmove = 0
                Print #2, "BOARD OFF"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Print #2, "MOVE "; actmove
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Print #2, "BOARD ON"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
                Sleep 1500, 1
                Print #2, "NORMAL"
                Do
                    dat = Input(1, #2)
                Loop Until dat = Chr(&H17)
            End If

            ' Input move
            Print #2, "ANALYSIS"
            Do
                dat = Input(1, #2)
            Loop Until dat = Chr(&H17)
            firstmove = 0 : bestmove = ""
            Do
                cdata = ""
                Do
                    dat = Input(1, #2)
                    ' Eliminate capture moves for easier evaluation
                    If dat = "x" Then dat = "-"
                    cdata = cdata + dat
                Loop Until dat = Chr(&H0D) Or dat = Chr(&H0A) Or dat = Chr(&H17)
            Loop Until Instr(cdata, "-") > 0 And Instr(cdata, ":") > 0
            n = Instr(cdata, "-")
            If Mid(cdata, n - 1, 3) = "O-O" Then
                If black = 0 Then
                    bestmove = "e1g1"
                Else
                    bestmove = "e8g8"
                End If
                If Mid(cdata, n - 1, 5) = "O-O-O" Then
                    If black = 0 Then
                        bestmove = "e1c1"
                    Else
                        bestmove = "e8c8"
                    End If
                End If
            Else
                bestmove = Mid(cdata, n - 2, 2) + Mid(cdata, n + 1, 2)
            End If
            n = Instr(cdata, "/ep") : If n = 0 Then n = Instr(cdata, "/EP")
            If n > 0 Then cdata = Left(cdata, n - 1) + Mid(cdata, n + 3)
            n = Instr(cdata, "/")
            If n > 0 Then bestmove = bestmove + LCase(Mid(cdata, n + 1, 1))
            Print "bestmove "; bestmove

        End If

Loop Until indata = "quit"

' Close open files and release COM port
Print #2, "CLOSE"
Close
End

Sub setuppos(ByRef fenstr As String)
    Dim As String tmp, b, col2move, piece, dat
    Dim As Short i, j, x, k
    i = InStr(1, fenstr, " ")
    If i = 0 Then
        Print "fen error"
        Exit Sub    ' wrong fen
    Else
        b = Left(fenstr, i - 1)	  ' positions
        k = i + 1
    End If
    i = InStr(k, fenstr, " ")
    If i = 0 Then
        Print "fen error"
        Exit Sub
    Else
        tmp = Mid(fenstr, i - 1, 1)	  ' color to move
        If tmp <> "w" And tmp <> "b" Then Print "fen error" : Exit Sub
        If tmp = "w" Then col2move = "WHITE " : black = 0
        If tmp = "b" Then  col2move = "BLACK " : black = 1
    End If

    ' switch to setup mode and clear board
    Print #2, "SETUP"
    Do
        dat = Input(1, #2)
    Loop Until dat = Chr(&H17)
    Print #2, "FUNCTION NEW"
    Do
        dat = Input(1, #2)
    Loop Until dat = Chr(&H17)

    ' convert fen postions to OSA positions and send each row
    k = 1 : b = b + "/"
    FOR i = 8 TO 1 Step -1      ' all rows
        x = 1                   ' set A-column
        tmp = ""
        Do
            If Asc(Mid(b, k, 1)) > 65 And Asc(Mid(b, k, 1)) < 83 Then	' big letter (white)
               Select Case Mid(b, k, 1)
                Case "P"
                    piece = "WP"
                Case "K"
                    piece = "WK"
                Case "Q"
                    piece = "WQ"
                Case "N"
                    piece = "WN"
                Case "B"
                    piece = "WB"
                Case "R"
                    piece = "WR"
               End Select
               tmp = tmp + piece + Chr(96 + x) + Chr(48 + i) + ","
               x = x + 1
            End If
            If Asc(Mid(b, k, 1)) > 97 And Asc(Mid(b, k, 1)) < 115 Then	' small letter (black)
               Select Case Mid(b, k, 1)
                Case "p"
                    piece = "BP"
                Case "k"
                    piece = "BK"
                Case "q"
                    piece = "BQ"
                Case "n"
                    piece = "BN"
                Case "b"
                    piece = "BB"
                Case "r"
                    piece = "BR"
               End Select
               tmp = tmp + piece + Chr(96 + x) + Chr(48 + i) + ","
               x = x + 1
            End If
            If Asc(Mid(b, k, 1)) > 48 And Asc(Mid(b, k, 1)) < 57 Then	' digits (empty fields)
               For j = 1 To ValUInt(Mid(b, k, 1))
                    x = x + 1
               Next j
            End If 
            k = k + 1
        Loop Until Mid(b, k, 1) = "/"
        If Right(tmp, 1) = "," Then tmp = Left(tmp, Len(tmp) - 1)
        If tmp <> "" Then
            Print #2, "POSI "; col2move; tmp
            Do
                dat = Input(1, #2)
            Loop Until dat = Chr(&H17)
            Sleep 250, 1
        End If
        k = k + 1
    NEXT i

    ' switch back to normal mode
    Print #2, "BOARD OFF"
    Do
        dat = Input(1, #2)
    Loop Until dat = Chr(&H17)
    Print #2, "NORMAL"
    Do
        dat = Input(1, #2)
    Loop Until dat = Chr(&H17)
End Sub
