''
'' UCI-Monitor interface for Saitek-OSA-Interace (COM port)
'' Optimized for Arena
'' V1.1
''
'' (C) 2012 by Achim Pietig <achim@pietig.com>
'' License:
''
'' Redistribution and use in source and binary forms, with or without
'' modification, are permitted provided that the following conditions
'' are met:
''
'' 1. Redistributions of source code must retain the above copyright
''    notice, this list of conditions and the following disclaimer.
'' 2. Redistributions in binary form must reproduce the above copyright
''    notice, this list of conditions and the following disclaimer in the
''    documentation and/or other materials provided with the distribution.
'' 3. The name of the author may not be used to endorse or promote products
''    derived from this software without specific prior written permission.
''
'' Changes to this license can be made only by the copyright author with
'' explicit written consent.
''
'' THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
'' IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
'' OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
'' IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
'' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
'' NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
'' DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
'' THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
'' (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
'' THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

' Lib for Function FileExists
#include "file.bi"

'
'   Main program
'

DIM As UShort n, i, black
DIM As String indata, dat, cdata, bestmove, pv
DIM As String actCOM, actBaud, oldpv, bname, bmodul
Dim Shared nParams As UByte
ReDim Shared Param(1 To 2) As String
' Sets default command line parameters
actCOM = "COM1:"
actBaud = "1200"

' Checking for INI file
If FileExists("osa.ini") Then
    Open "osa.ini" for Input As #4
    Do Until EOF(4)
        Line Input #4, indata
        indata = Trim(indata) : dat = ""
        ' Strip off blanks
        For i = 1 to Len(indata)
            If Mid(indata, i, 1) <> " " Then dat = dat + Mid(indata, i, 1)
        Next i
        If LCase(Left(dat, 4)) = "com=" Then actCOM = Mid(dat, 5)
        If LCase(Left(dat, 5)) = "baud=" Then actBaud = Mid(dat, 6)
        If LCase(Left(dat, 5)) = "name=" Then bname = Mid(dat, 6)
        If LCase(Left(dat, 6)) = "modul=" Then bmodul = Mid(dat, 7)
    Loop
    Close #4                        
End If

' Checking for command line parameters
nParams = 0
While Command(nParams + 1) <> ""
    nParams = nParams + 1
Wend
If nParams > 0 Then
    ReDim Param(1 To nParams) As String
    For i = 1 To nParams
        Param(i) = Command(i)
    Next i
End If

' Help
If UCase(Param(1)) = "/?" Or UCase(Param(1)) = "/H" Then
    Print
    Print "OSA-Monitor for Scisys/Saitek chess boards"
    Print "(C) 2012 by Achim Pietig, Version 1.1"
    Print
    Print "Usage: OSA-Monitor [/?] Or [/h]"
    Print "       OSA-Monitor [COMx:] [baud]"
    Print "Call without parameters sets COM1: with 1200 baud"
    Print "Board shall be set to corresponding baud rate before!"
    Print
    Print "e.g.   OSA-Monitor             (COM1: with 1200 baud)"
    Print "       OSA-Monitor COM2:       (COM2: with 1200 baud)"
    Print "       OSA-Monitor COM3: 9600  (COM3: with 9600 baud)"
    End
End If

' Init program parameters
If nParams > 0 Then actCOM = UCase(Param(1))
If nParams > 1 Then actBaud = Param(2)

' Opens related COM port with board specific values
' OSA cable supports TxD, RxD only. All hardware specific handshakes are disabled.
Open Cons For Input As #1
If Open Com (actCOM + actbaud + ",N,8,1,CS0,DS0,CD0,RS,BIN" For Binary As #2) <> 0 Then
    Print "Cannot initialise comport"
    Close
    End
End If

' Sets OSA to english language for all following commands
Print #2, "ENGLISH"
' There is no response, so wait 250 ms to give the board some time before sending the next command
Sleep 250,1

Print #2, "OPEN"
Do
    dat = Input(1, #2)
    If Asc(dat) > 31 And dat <> ">" Then cdata = cdata + dat
Loop Until dat = Chr(&H17)
' Open command returns an initial string from the board with OSA version and actual baud rate
' The answer is shown as info string in Arena (engine debug window)
Print "info string "; Trim(cdata)

' Switch on automatic responses of the board for monitoring
Print #2, "SENDINFO ON"
Do
    dat = Input(1, #2)
Loop Until dat = Chr(&H17)

' Program loop
Do
        ' Waits for an Arena UCI command
        Line Input #1, indata
        indata = Trim(indata)
        cdata = "" : dat = "" : bestmove = ""

        ' Answer to Arena in case of 'uci'
        If indata = "uci" Then
            If bname + bmodul = "" Then
                Print "id name OSA-Monitor V1.1"
                Print "id author Achim Pietig"
            Else
                Print "id name "; bname;
                If bmodul <> "" Then Print " with "; bmodul;
                Print
            End If
            ' Typical values for chess computers with own book
            Print "option name OwnBook type check default true"
            Print "uciok"
        End If

        ' Answer to Arena in case of 'isready'
        If indata = "isready" Then
            Print "readyok"
        End If

        ' Action in case of 'go' command from Arena, response is 'bestmove' with the move from the board
        If Left(indata, 2) = "go" Then
            oldpv = ""
            Do
                bestmove = "" : cdata = "" : black = 0
                Do
                    dat = Input(1, #2)
                    cdata = cdata + dat
                Loop Until dat = Chr(&H0A) Or  dat = Chr(&H17)
				' Strip off CR/LF and/or &H17
				If Right(cdata, 1) = Chr(&H0A) Then cdata = Left(cdata, Len(cdata) - 2)
				If Right(cdata, 1) = Chr(&H17) Then cdata = Left(cdata, Len(cdata) - 1)
                cdata = Trim(cdata)
                ' Eliminate capture moves for easier evaluation
                n = Instr(cdata, "x")
                If n > 0 Then Mid(cdata , n, 1) = "-"
				cdata = Trim(cdata)
                ' Conditions for program termination
                If Instr(cdata, "1-0") > 0 Or Instr(cdata, "0-1") > 0 Or Instr(cdata, ".5-.5") > 0 Or Instr(cdata, "New ") > 0 Then indata = "quit"
				
                If Left(cdata, 8) = "Sendinfo" Then
                    dat = Mid(cdata, 29)
                    ' Actual line is stored in pv
                    pv = dat
                    ' If actual line differs from the previous, it is send to Arena as 'info'
                    If pv <> oldpv Then
                        ' Check for opening book info and skip it
                        If Trim(Mid(cdata, 18, 2)) = "1" And Abs(ValInt(Trim(Mid(cdata, 21, 6)))) > 100 And Len(pv) < 11 Then
                            ' Do nothing
                        Else
                            ' Send info
                            Print "info depth "; Trim(Mid(cdata, 18, 2)); " score cp "; Trim(Mid(cdata, 21, 6)); " pv "; pv
                            oldpv = pv
                        End If
                    End If
					cdata = ""
                End If

                If Instr(cdata, "Ready") Then
					cdata = ""
					Do
						dat = Input(1, #2)
						cdata = cdata + dat
					Loop Until dat = Chr(&H0A) Or dat = Chr(&H17)
                    ' Strip off CR/LF and/or &H17
                    If Right(cdata, 1) = Chr(&H0A) Then cdata = Left(cdata, Len(cdata) - 2)
                    If Right(cdata, 1) = Chr(&H17) Then cdata = Left(cdata, Len(cdata) - 1)
                    cdata = Trim(cdata)
					' strip off time-stamp and unnecessary blanks
					cdata = Trim(Left(cdata, Len(cdata) - 5))
                    ' Eliminate capture moves for easier evaluation
                    n = Instr(cdata, "x")
                    If n > 0 Then Mid(cdata , n, 1) = "-"
					' Strip off move number
					n = Instr(cdata, ".")
					If n > 0 Then
						cdata = Trim(Mid(cdata, n + 1))
					Else
						black = 1
					End If	
					bestmove = cdata
                    cdata = ""
                End If

                If Instr(cdata, "-") > 0 And Instr(cdata, ":") > 0 Then
                    ' Strip off CR/LF and/or &H17
                    If Right(cdata, 1) = Chr(&H0A) Then cdata = Left(cdata, Len(cdata) - 2)
                    If Right(cdata, 1) = Chr(&H17) Then cdata = Left(cdata, Len(cdata) - 1)
                    cdata = Trim(cdata)
					' strip off time-stamp and unnecessary blanks
					cdata = Trim(Left(cdata, Len(cdata) - 5))
					' Strip off move number
					n = Instr(cdata, ".")
					If n > 0 Then
						cdata = Trim(Mid(cdata, n + 1))
					Else
						black = 1
					End If	
					bestmove = cdata
                    cdata = ""
                End If
				
            Loop Until Len(bestmove) > 2

			' Format bestmove to UCI
            ' Check for check, checkmate and en passant
            n = Instr(bestmove, "++")
            If n > 0 Then
                bestmove = Left(bestmove, n - 1) + Mid(bestmove , n + 2)
                indata = "quit"
            End If
            n = Instr(bestmove, "+")
            If n > 0 Then bestmove = Left(bestmove, n - 1) + Mid(bestmove , n + 1)
            n = Instr(bestmove, "/ep") : If n = 0 Then n = Instr(bestmove, "/EP")
            If n > 0 Then bestmove = Left(bestmove, n - 1) + Mid(bestmove , n + 3)
            n = Instr(bestmove, "/")
            If n > 0 Then bestmove = Left(bestmove, n - 1) + LCase(Mid(bestmove, n + 1, 1))
            If Instr(bestmove, "O-O-O") Then
                If black = 0 Then
                    bestmove = "e1c1"
                Else
                    bestmove = "e8c8"
                End If
			End If
			If Instr(bestmove, "O-O") Then
				If black = 0 Then
					bestmove = "e1g1"
				Else
					bestmove = "e8g8"
				End If
			End If
            n = Instr(bestmove, "-")
			If n > 0 Then bestmove = Mid(bestmove, n - 2, 2) + Mid(bestmove, n + 1)
            Print "bestmove "; bestmove
        End If

' Action on Arena command 'quit'
Loop Until indata = "quit"

' Flush COM buffer
' Clears the buffer from unnecessary bytes
While LOC(2) > 0
    dat = Input(LOC(2), #2)
Wend

' Close open files and release COM port
Print #2, "CLOSE"
Close
End
